package com.gic.cloud.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.cloud.constants.FunctionEnum;
import com.gic.cloud.dto.FunctionDTO;
import com.gic.cloud.dto.FunctionModuleDTO;
import com.gic.cloud.dto.UserDTO;
import com.gic.cloud.entity.TabSysFunction;
import com.gic.cloud.entity.TabSysFunctionModule;
import com.gic.cloud.entity.TabSysUser;
import com.gic.cloud.service.FunctionApiService;
import com.gic.cloud.service.FunctionModuleService;
import com.gic.cloud.service.FunctionService;
import com.gic.cloud.service.UserService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.error.ErrorCode;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Service("functionApiService")
public class FunctionApiServiceImpl implements FunctionApiService{
    @Autowired
    private FunctionService functionService;
    @Autowired
    private UserService userService;
    @Autowired
    private FunctionModuleService functionModuleService;

    @Override
    public ServiceResponse<Integer> saveFunction(FunctionDTO dto) {
        dto.setFunctionId(null);
        ServiceResponse validParamResult = validParam(dto);
        if (!validParamResult.isSuccess()) {
            return ServiceResponse.failure(validParamResult.getCode(), validParamResult.getMessage());
        }
        return ServiceResponse.success(functionService.saveFunction(dto));
    }

    @Override
    public ServiceResponse<Void> editFunction(FunctionDTO dto) {
        ServiceResponse validParamResult = validParam(dto);
        if (!validParamResult.isSuccess()) {
            return ServiceResponse.failure(validParamResult.getCode(), validParamResult.getMessage());
        }
        functionService.editFunction(dto);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> deleteFunction(Integer functionId) {
        functionService.deleteFunction(functionId);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Page<FunctionDTO>> pageFunction(Integer enterpriseId, String functionName,
                                                           Integer currentPage, Integer pageSize) {
        com.github.pagehelper.Page page = functionService.pageFunction(enterpriseId, functionName, currentPage, pageSize);
        Page<FunctionDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page, FunctionDTO.class);
        if (resultPage != null) {
            List<FunctionDTO> list = resultPage.getResult();
            if (CollectionUtils.isNotEmpty(list)) {
                Stream<FunctionDTO> stream = list.stream();
                //获取授权人数数据
                List<UserDTO> userList = userService.countGroupByFunctionAuthId(enterpriseId, stream.map(e -> e.getFunctionId())
                        .collect(Collectors.toList()));
                if (CollectionUtils.isNotEmpty(userList)) {
                    Map<String, Integer> map = userList
                            .stream()
                            .collect(Collectors.toMap(e -> e.getFunctionAuthId().toString(), e -> e.getMemberCount()));
                    list.forEach(e -> e.setMemberCount(map.get(e.getFunctionId().toString())));
                }
            }
            return ServiceResponse.success(resultPage);
        }
        return ServiceResponse.success(new Page<>());
    }

    @Override
    public ServiceResponse<List<FunctionDTO>> listFunction(Integer enterpriseId, String functionName) {
        return ServiceResponse.success(Optional.ofNullable(EntityUtil.changeEntityListNew(FunctionDTO.class,
                functionService.listFunction(enterpriseId, functionName)))
                .orElse(Collections.EMPTY_LIST));
    }

    @Override
    public ServiceResponse<FunctionDTO> getFunction(Integer functionId) {
        return ServiceResponse.success(EntityUtil.changeEntityNew(FunctionDTO.class, functionService.getFunction(functionId)));
    }

    @Override
    public ServiceResponse<List<FunctionModuleDTO>> getFunctionByUserId(Integer userId) {
        TabSysUser user = userService.getUser(userId);
        if (user == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "用户不存在");
        }
        TabSysFunction record = functionService.getFunction(user.getFunctionAuthId());

        if (record != null) {
            List<TabSysFunctionModule> moduleList = functionModuleService.listFunctionModule(null);
            Map<String, TabSysFunctionModule> map = moduleList.stream().collect(Collectors.toMap(e -> e.getFunctionModuleCode(), e -> e));
            return ServiceResponse.success(Arrays.stream(record.getFunctionComponent()
                    .split("_"))
                    .filter(e -> StringUtils.isNotBlank(e))
                    .map(e -> EntityUtil.changeEntityNew(FunctionModuleDTO.class, map.get(e)))
                    .collect(Collectors.toList()));
        }
        return ServiceResponse.success(Collections.emptyList());
    }

    private ServiceResponse<Void> validParam(FunctionDTO dto) {
        if (dto.getFunctionId() != null) {
            TabSysFunction record = functionService.getFunction(dto.getFunctionId());
            if (record == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "功能权限组ID错误，查无数据");
            }
            dto.setEnterpriseId(record.getEnterpriseId());
        }
        Integer enterpriseId = dto.getEnterpriseId();
        if (enterpriseId == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID不能为空");
        }
        String functionName = dto.getFunctionName();
        if (StringUtils.isBlank(functionName)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "功能权限组名称不能为空");
        }
        if (functionService.isRepeatFunctionName(enterpriseId, dto.getFunctionId(), functionName)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "功能权限组名称重复");
        }
        String functionComponent = dto.getFunctionComponent();
        if (StringUtils.isBlank(functionComponent)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "功能组件不能为空");
        }
        //一个流只能用一次
        Stream<String> stream = Arrays.stream(functionComponent.split(","));
        Stream<String> stream1 = Arrays.stream(functionComponent.split(","));
        if (stream.anyMatch(e -> !FunctionEnum.isRightCode(e))) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "功能组件值非法");
        }
        StringBuilder temp = new StringBuilder("_");
        stream1.sorted().forEach(e -> {
            temp.append(e).append("_");
        });
        dto.setFunctionComponent(temp.toString());
        return ServiceResponse.success();
    }
}
